/* $Id: print.c,v 1.3 2004/02/23 04:36:01 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: print.c
 *            DESCRIPTION: print routines for XNotesPlus 
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPPrintNote
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: print.c,v $
 * Revision 1.3  2004/02/23 04:36:01  mjhammel
 * Calendar complete for beta 1.
 * Initial documentation updates for 3.6.
 * Fixed bug in FindText.c - accept button should be "gshell.search", not
 * "gshell.accept".
 *
 * Revision 1.2  2004/02/20 04:49:24  mjhammel
 * Added CUPS support for printing.
 * Fixed monthly display in appt's to only highlight date for current month.
 * Changed notebook layout in appt's.
 *
 * Revision 1.1.1.1  2002/01/10 18:38:05  mjhammel
 * Initial checkin
 *
 *
 *========================================================================*/
#define XP_PRINT_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#ifdef USE_CUPS
#include <cups/cups.h>
#endif

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
int XPPrintFile();


/* === Private routine prototypes === */


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*toplevel;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPPrintNote
 *	Prototype:	XPPrintNote(GtkWidget *widget, int index)
 *					
 *	Description:
 * Prints a note.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPPrintNote(
	int			index
)
{
	StickyNote	*pn;
	FILE			*fp;
	int			len;
	char			tmpfile[512];
	char			cmd[1024];
	char			buf[1024];
	int			rc;

	/*
	 * Find the note we're printing.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Retrieve the current text from the note.
	 * How do we do this?
	 */

	/*
	 * Make sure there is something to print.
	 */
	if (pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	len = strlen(pn->pn_text);
	if ( strlen(pn->pn_text) == 0 )
	{
		XPErrPopUp(XP_S_NO_TEXT_TO_PRINT);
		return;
	}

	/*
	 * Open a temporary file
	 */
	sprintf(tmpfile,"%s/.xp%d", app_res.note_dir, (int)getpid());
	if ((fp = fopen(tmpfile, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile);
		sprintf(cmd, XP_S_TEMP_OPEN_FAIL, tmpfile);
		XPErrPopUp(cmd);
		return;
	}

	/*
	 * Write note title to temp file.
	 */
	len = strlen(pn->pn_name->str);
	if (len)
	{
		sprintf(buf, "Note: ");
		fwrite(buf, sizeof(char), strlen(buf), fp);
		fwrite(pn->pn_name->str, sizeof(char), len, fp);
		sprintf(buf, "\n");
		fwrite(buf, sizeof(char), strlen(buf), fp);
		sprintf(buf, "---------------------------------------------------\n");
		fwrite(buf, sizeof(char), strlen(buf), fp);
	}
	else
	{
		sprintf(buf, XP_S_DEFAULT_NOTE_TITLE);
		fwrite(buf, sizeof(char), strlen(buf), fp);
	}

	/*
	 * Write note text to temp file.
	 */
	len = strlen(pn->pn_text);
	if (len)
		fwrite(pn->pn_text, sizeof(char), len, fp);
	else
	{
		XPErrPopUp(XP_S_NO_TEXT_TO_PRINT);
		return;
	}
	fclose(fp);

	if ( pn->pn_name == NULL )
	{
		sprintf(buf, "Note %d", pn->pn_index);
		printf("buf: %s\n", buf);
		rc = XPPrintFile(tmpfile, buf);
	}
	else
		rc = XPPrintFile(tmpfile, pn->pn_name->str);

	if ( rc == XP_E_SUCCESS )
	{
		XPMsgWindow(
			XP_S_NOTE_PRINTED, 
			XP_INFO_TYPE | XP_MSG_NOACCEPT | XP_MSG_NOHELP, NULL,
			NULL, NULL, NULL, NULL, NULL);
	}

	/*
	 * clean up the temporary file
	 */
	unlink(tmpfile);

}


/*========================================================================
 *	Name:			XPPrintFile
 *	Prototype:	XPPrintFile(char *)
 *					
 *	Description:
 * Prints a file.  Uses CUPS if compiled to do so, otherwise uses
 * configured print command.
 *========================================================================*/
int
XPPrintFile(
	char	*filename,
	char	*title
)
{
	char			cmd[1024];

#ifdef USE_CUPS

	int			jobid;
	int			i;
	int			num_dests;
	cups_dest_t	*dests;
	int			error;

	error = 0;
	num_dests = cupsGetDests(&dests);
	for(i=0; i<num_dests; i++)
	{
		if ( dests[i].is_default )
		{
			jobid = cupsPrintFile(dests[i].name, filename, title, 0, NULL);
			if ( jobid == 0 )
			{
				error = 1;
				sprintf(cmd,"Print failed: %s", ippErrorString(cupsLastError()));
				XPErrPopUp(cmd);
			}
		}
	}
	cupsFreeDests(num_dests, dests);
	if ( error )
		return(XP_E_FAILURE);
	else
		return(XP_E_SUCCESS);

#else

#ifdef DEBUG
	char			*fname="XPPrintNote";
	int			rc;
#endif

	/*
	 * Use the printcmd resource to determine how to print the note
	 */
	sprintf (cmd, app_res.print_cmd, filename );
#ifdef DEBUG
	DBGPrintf(DBG_INFO,("Command used to print the note: %s\n", cmd));
	rc = system ( cmd );
	switch ( rc )
	{
		case 127:
			sprintf(cmd,"%s failed: errno=%d", cmd, errno);
			XPErrPopUp(cmd);
			return(XP_E_FAILURE);
			break;
			
		case -1:
			sprintf(cmd,"%s failed: errno=%d", cmd, rc);
			XPErrPopUp(cmd);
			return(XP_E_FAILURE);
			break;
			
		default:
			XPMsgWindow(
				XP_S_NOTE_PRINTED, 
				XP_INFO_TYPE | XP_MSG_NOACCEPT | XP_MSG_NOHELP, NULL,
				NULL, NULL, NULL, NULL, NULL);
			return(XP_E_SUCCESS);
			break;
	}
#else
	system ( cmd );
	return(XP_E_SUCCESS);
#endif

#endif /* USE_CUPS */
}
